@@ -748,6 +748,62 @@ void ramanujan_tau(mpz_t res, mpz_t n)
   mpz_clear(t4); mpz_clear(t3); mpz_clear(t2); mpz_clear(t1); mpz_clear(t);
 }
 
+int is_semiprime(mpz_t n)
+{
+  int ret;
+  UV div, lim = 6000;
+  mpz_t t;
+
+  if (mpz_cmp_ui(n,6) < 0)
+    return (mpz_cmp_ui(n,4) == 0);
+
+  mpz_init(t);
+
+  div = _GMP_trial_factor(n, 2, lim);
+  if (div > 0) {
+    mpz_divexact_ui(t, n, div);
+    ret = _GMP_is_prime(t);
+    mpz_clear(t);
+    return !!ret;
+  }
+  /* No small divisors */
+  if (_GMP_BPSW(n))
+    { mpz_clear(t); return 0; }
+  if (mpz_ui_pow_ui(t,lim,3), mpz_cmp(n,t) < 0)
+    { mpz_clear(t); return 1; }
+  /* Number is composite, isn't tiny, and has no small divisors */
+  if (    0
+       || _GMP_pbrent_factor(n, t, 1, 15000)
+       || _GMP_pminus1_factor(n, t, 50000, 500000)
+       || _GMP_ECM_FACTOR(n, t,     800, 10)
+       || _GMP_ECM_FACTOR(n, t,    8000, 20)
+       || _GMP_ECM_FACTOR(n, t,   80000, 40)
+       || _GMP_ECM_FACTOR(n, t,  320000, 40)
+       || _GMP_ECM_FACTOR(n, t, 1000000, 80)
+     ) {
+    ret = _GMP_BPSW(t);
+    if (ret) {
+      mpz_divexact(t, n, t);
+      ret = _GMP_BPSW(t);
+    }
+    mpz_clear(t);
+    return !!ret;
+  }
+  /* No luck finding a small factor.  Do it the hard way. */
+  {
+    mpz_t* factors;
+    int* exponents;
+    int nfactors, i, j;
+
+    nfactors = factor(n, &factors, &exponents);
+    for (i = 0, j = 0; i < nfactors; i++)
+      j += exponents[i];
+    clear_factors(nfactors, &factors, &exponents);
+    mpz_clear(t);
+    return (j == 2);
+  }
+}
+
 
 /*****************************************************************************/
 
@@ -925,10 +981,9 @@ int _GMP_prho_factor(mpz_t n, mpz_t f, UV a, UV rounds)
       mpz_mul(U, U, U);  mpz_add_ui(U, U, a);  mpz_tdiv_r(U, U, n);
       mpz_mul(V, V, V);  mpz_add_ui(V, V, a);  mpz_tdiv_r(V, V, n);
       mpz_mul(V, V, V);  mpz_add_ui(V, V, a);  mpz_tdiv_r(V, V, n);
-      if (mpz_cmp(U, V) >= 0)  mpz_sub(f, U, V);
-      else                     mpz_sub(f, V, U);
+      mpz_sub(f, U, V);
       mpz_mul(m, m, f);
-      mpz_tdiv_r(m, m, n);
+      if ((i%4) == ((inner-1)%4)) mpz_tdiv_r(m, m, n);
     }
     mpz_gcd(f, m, n);
     if (!mpz_cmp_ui(f, 1))
@@ -977,10 +1032,9 @@ int _GMP_pbrent_factor(mpz_t n, mpz_t f, UV a, UV rounds)
       mpz_set(saveXi, Xi);
       for (i = 0; i < dorounds; i++) {
         mpz_mul(t, Xi, Xi);  mpz_add_ui(t, t, a);  mpz_tdiv_r(Xi, t, n);
-        if (mpz_cmp(Xi, Xm) >= 0)  mpz_sub(f, Xi, Xm);
-        else                       mpz_sub(f, Xm, Xi);
-        mpz_mul(t, m, f);
-        mpz_tdiv_r(m, t, n);
+        mpz_sub(f, Xm, Xi);
+        mpz_mul(m, m, f);
+        if ((i%4) == ((dorounds-1)%4)) mpz_tdiv_r(m, m, n);
       }
       rleft -= dorounds;
       rounds -= dorounds;
@@ -998,18 +1052,18 @@ int _GMP_pbrent_factor(mpz_t n, mpz_t f, UV a, UV rounds)
       mpz_set(Xi, saveXi);
       do {
         mpz_mul(t, Xi, Xi);  mpz_add_ui(t, t, a);  mpz_tdiv_r(Xi, t, n);
-        if (mpz_cmp(Xi, Xm) >= 0)  mpz_sub(f, Xi, Xm);
-        else                       mpz_sub(f, Xm, Xi);
+        mpz_sub(f, Xm, Xi); if (mpz_sgn(f) < 0) mpz_add(f,f,n);
         mpz_gcd(f, f, n);
       } while (!mpz_cmp_ui(f, 1) && r-- != 0);
-      if ( (!mpz_cmp_ui(f, 1)) || (!mpz_cmp(f, n)) )  break;
     }
-    mpz_clear(Xi); mpz_clear(Xm); mpz_clear(m); mpz_clear(saveXi); mpz_clear(t);
-    return 1;
+    break;
   }
   mpz_clear(Xi); mpz_clear(Xm); mpz_clear(m); mpz_clear(saveXi); mpz_clear(t);
-  mpz_set(f, n);
-  return 0;
+  if (!mpz_cmp_ui(f, 1) || !mpz_cmp(f, n)) {
+    mpz_set(f, n);
+    return 0;
+  }
+  return 1;
 }
 
 int _GMP_pminus1_factor(mpz_t n, mpz_t f, UV B1, UV B2)
@@ -1601,3 +1655,39 @@ UV power_factor(mpz_t n, mpz_t f)
   }
   return (k == 1) ? 0 : k;
 }
+
+static int numcmp(const void *av, const void *bv)
+  { return mpz_cmp(*(const mpz_t*)av, *(const mpz_t*)bv); }
+
+mpz_t * divisor_list(int *num_divisors, mpz_t n)
+{
+  mpz_t *factors, *divs, mult;
+  int nfactors, ndivisors, i, j, k, count, *exponents;
+
+  nfactors = factor(n, &factors, &exponents);
+  ndivisors = 1 + ((nfactors > 0) ? exponents[0] : 0);
+  for (i = 1; i < nfactors; i++)
+    ndivisors *= (exponents[i] + 1);
+
+  mpz_init(mult);
+  New(0, divs, ndivisors, mpz_t);
+  mpz_init_set_ui(divs[0], 1);
+  for (count = 1, k = 0; k < nfactors; k++) {
+    int scount = count;
+    mpz_set_ui(mult, 1);
+    for (j = 0; j < exponents[k]; j++) {
+      mpz_mul(mult, mult, factors[k]);
+      for (i = 0; i < scount; i++) {
+        mpz_init(divs[count]);
+        mpz_mul(divs[count], divs[i], mult);
+        count++;
+      }
+    }
+  }
+  clear_factors(nfactors, &factors, &exponents);
+
+  qsort(divs, ndivisors, sizeof(mpz_t), numcmp);
+
+  *num_divisors = ndivisors;
+  return divs;
+}
