@@ -20,6 +20,7 @@ extern void bernfrac(mpz_t num, mpz_t den, mpz_t n);
 extern void harmfrac(mpz_t num, mpz_t den, mpz_t n);
 extern void stirling(mpz_t r, unsigned long n, unsigned long m, UV type);
 extern void binomial(mpz_t r, UV n, UV k);
+extern void partitions(mpz_t npart, UV n);
 
 extern UV   is_power(mpz_t n, UV a);
 
@@ -31,6 +32,7 @@ extern char* bernreal(mpz_t zn, unsigned long prec);
 extern char* harmreal(mpz_t zn, unsigned long prec);
 extern char* zetareal(mpf_t r, unsigned long prec);
 extern char* riemannrreal(mpf_t r, unsigned long prec);
+extern char* lambertwreal(mpf_t r, unsigned long prec);
 
 extern UV* sieve_primes(mpz_t low, mpz_t high, UV k, UV *rn);
 extern UV* sieve_twin_primes(mpz_t low, mpz_t high, UV twin, UV *rn);
