@@ -5,7 +5,7 @@ use Carp qw/croak confess carp/;
 
 BEGIN {
   $Math::Prime::Util::GMP::AUTHORITY = 'cpan:DANAJ';
-  $Math::Prime::Util::GMP::VERSION = '0.42';
+  $Math::Prime::Util::GMP::VERSION = '0.43';
 }
 
 # parent is cleaner, and in the Perl 5.10.1 / 5.12.0 core, but not earlier.
@@ -86,7 +86,12 @@ our @EXPORT_OK = qw(
                      Pi
                      todigits
                      random_prime random_nbit_prime random_ndigit_prime
+                     random_strong_prime
+                     random_maurer_prime random_shawe_taylor_prime
+                     random_maurer_prime_with_cert
+                     random_shawe_taylor_prime_with_cert
                      seed_csprng is_csprng_well_seeded
+                     urandomb urandomr
                    );
                    # Should add:
                    # nth_prime
@@ -181,7 +186,7 @@ __END__
 
 =encoding utf8
 
-=for stopwords Möbius Deléglise Bézout gcdext vecsum vecprod moebius totient liouville znorder znprimroot bernfrac bernreal harmfrac harmreal stirling zeta riemannr lambertw lucasu lucasv OpenPFGW gmpy2 nonresidue chinese tuplets sqrtmod addmod mulmod powmod divmod superset sqrtint rootint todigits
+=for stopwords Möbius Deléglise Bézout gcdext vecsum vecprod moebius totient liouville znorder znprimroot bernfrac bernreal harmfrac harmreal stirling zeta riemannr lambertw lucasu lucasv OpenPFGW gmpy2 nonresidue chinese tuplets sqrtmod addmod mulmod powmod divmod superset sqrtint rootint todigits urandomb urandomr
 
 =head1 NAME
 
@@ -190,7 +195,7 @@ Math::Prime::Util::GMP - Utilities related to prime numbers and factoring, using
 
 =head1 VERSION
 
-Version 0.42
+Version 0.43
 
 
 =head1 SYNOPSIS
@@ -914,6 +919,20 @@ Returns a randomly selected prime of exactly C<n> bits.
 C<undef> is returned if C<n> is less than C<2>.
 The returned prime has passed the C<is_prob_prime> (extra strong BPSW) test.
 
+=head2 random_strong_prime
+
+  say "random 512-bit strong prime: ", random_strong_prime(512);
+
+Returns a randomly selected strong prime of exactly C<n> bits.
+C<n> must be at least C<256>.
+The returned prime has passed the C<is_prob_prime> (extra strong BPSW) test.
+
+Given the returned prime C<p>, C<p+1>, C<q=p-1>, and C<q-1> will all have
+a large factor.  This makes using factoring methods such as p-1 and p+1 much
+harder.  Gordon's algorithm is used.  The value of using strong primes is
+questionable over proper random primes when the number of bits is
+at least 1024.
+
 =head2 random_ndigit_prime
 
   say "random 200-digit prime: ", random_ndigit_prime(200);
@@ -933,6 +952,44 @@ The returned prime has passed the C<is_prob_prime> (extra strong BPSW) test.
 The random prime functions use the internal CSPRNG for randomness.  This
 is currently ISAAC-32.
 
+This corresponds to Mathematica's C<RandomPrime[{min,max}]> function.
+This is a superset of Pari's C<randomprime(n)> function, where our interval
+API is more useful for cryptographic functions.
+
+=head2 random_maurer_prime
+
+  say "random 512-bit proven prime: ", random_maurer_prime(512);
+
+Returns an n-bit proven prime using Ueli Maurer's FastPrime algorithm (1995).
+This results in uniform random selection of a proven prime, though with a
+small subset of the range not returned.
+
+C<undef> is returned if C<n> is less than C<2>.
+Internally the extra strong BPSW test has additionally been run on each
+intermediate and the final result, as a safety check.
+
+=head2 random_shawe_taylor_prime
+
+  say "random 512-bit proven prime: ", random_shawe_taylor_prime(512);
+
+Returns an n-bit proven prime using the Shawe-Taylor algorithm (1986) from
+section C.6 of FIPS 186-4, although using our CSPRNG rather than SHA-256.
+This is a slightly simpler and older (1986) method than Maurer's algorithm.
+It is a bit faster than Maurer's method but has a smaller subset of returned
+primes.
+
+C<undef> is returned if C<n> is less than C<2>.
+Internally the extra strong BPSW test has additionally been run on each
+intermediate and the final result, as a safety check.
+
+=head2 random_maurer_prime_with_cert
+
+Like L</random_maurer_prime> but also returns a string certificate.
+
+=head2 random_shawe_taylor_prime_with_cert
+
+Like L</random_shawe_taylor_prime> but also returns a string certificate.
+
 
 =head2 lucasu
 
@@ -1706,6 +1763,28 @@ On startup the module will attempt to seed the CSPRNG from
 C</dev/urandom>, so this function will return true if that was
 successful, but false otherwise.
 
+=head2 urandomb
+
+  $n32 = urandomb(32);    # Classic irand32, returns a UV
+  $n   = urandomb(1024);  # Random integer less than 2^1024
+
+Given a number of bits C<b>, returns a random unsigned integer less than C<2^b>.
+The result will be uniformly distributed between C<0> and C<2^b-1> inclusive.
+
+This is similar to the GMP function C<mpz_urandomb>.
+
+=head2 urandomr
+
+  $n  = urandomr(100, 110);        # Random number [100,110]
+  $nb = urandomr(2**24,2**25-1);   # Random 25-bit number
+  $nd = urandomr(10**24,10**25-1); # Random 25-digit number
+
+Given values C<low> and C<high>, returns a uniform random unsigned integer
+in the range C<[low,high]>.  Both inputs must be non-negative.
+If C<low E<gt> high> then function will return C<undef>.
+Note that the range is inclusive, so C<low>, C<high>, and each integer
+between them have an equal probability of appearing.
+
 
 =head1 SEE ALSO
 
