@@ -15,6 +15,72 @@
 static const unsigned char sprimes[NSMALLPRIMES] = {2,3,5,7,11,13,17,19,23,29,31,37,41,43,47,53,59,61,67,71,73,79,83,89,97,101,103,107,109,113,127,131,137,139,149,151,157,163,167,173,179,181,191,193,197,199,211,223,227,229,233,239,241,251};
 
 
+/* 0 fail, 1 pass, -1 nothing.  Modifies base a */
+static int _preprocess_base(mpz_t n, mpz_t a)
+{
+  if (mpz_cmp_ui(a, 1) < 0)
+    croak("Base %ld is invalid", mpz_get_si(a));
+  if (mpz_cmp_ui(n, 3) <= 0)
+    return (mpz_cmp_ui(n, 2) >= 0);
+
+  if (mpz_cmp_ui(a, 2) > 0) {
+    if (mpz_cmp(a, n) >= 0) {
+      mpz_mod(a, a, n);
+      if (mpz_cmp_ui(a, 1) <= 0)
+        return mpz_sgn(a);
+    }
+  }
+  return -1;
+}
+
+int is_pseudoprime(mpz_t n, mpz_t a)
+{
+  mpz_t nm1;
+  int res;
+
+  if ((res = _preprocess_base(n, a)) >= 0)
+    return res;
+
+  mpz_init(nm1);
+  mpz_sub_ui(nm1, n, 1);
+  mpz_powm(nm1, a, nm1, n);
+  res = (mpz_cmp_ui(nm1, 1) == 0);
+  mpz_clear(nm1);
+  return res;
+}
+
+int is_euler_pseudoprime(mpz_t n, mpz_t a)
+{
+  mpz_t nm1, ap;
+  int res;
+
+  if (mpz_even_p(n))
+    return (mpz_cmp_ui(n,2) == 0);
+  if ((res = _preprocess_base(n, a)) >= 0)
+    return res;
+
+  mpz_init(ap);
+  if (mpz_gcd(ap, a, n), mpz_cmp_ui(ap, 1) != 0) {
+    mpz_clear(ap);
+    return 0;
+  }
+  mpz_init(nm1);
+  mpz_sub_ui(nm1, n, 1);
+  mpz_tdiv_q_2exp(ap, nm1, 1);
+  mpz_powm(ap, a, ap, n);
+
+  if (mpz_cmp_ui(ap, 1) && mpz_cmp(ap, nm1))
+    res = 0;
+  else if (mpz_kronecker(a, n) >= 0)
+    res = (mpz_cmp_ui(ap, 1) == 0);
+  else
+    res = (mpz_cmp(ap, nm1) == 0);
+
+  mpz_clear(nm1);
+  mpz_clear(ap);
+  return res;
+}
+
 static int mrx(/*destroyed*/mpz_t x, /*destroyed*/ mpz_t d, mpz_t n, UV s)
 {
   UV r;
@@ -143,7 +209,7 @@ int is_miller_prime(mpz_t n, int assume_grh)
 
 int miller_rabin_random(mpz_t n, UV numbases, char* seedstr)
 {
-  gmp_randstate_t* p_randstate = get_randstate();
+  gmp_randstate_t randstate;
   mpz_t t, base;
   UV i;
 
@@ -155,13 +221,14 @@ int miller_rabin_random(mpz_t n, UV numbases, char* seedstr)
 
   if (seedstr != 0) { /* Set the RNG seed if they gave us a seed */
     mpz_set_str(t, seedstr, 0);
-    gmp_randseed(*p_randstate, t);
+    gmp_randseed(randstate, t);
   }
 
   mpz_sub_ui(t, n, 3);
   for (i = 0; i < numbases; i++) {
-    mpz_urandomm(base, *p_randstate, t);  /* base = 0 .. (n-3)-1 */
-    mpz_add_ui(base, base, 2);            /* base = 2 .. n-2     */
+    if (seedstr != 0) mpz_urandomm(base, randstate, t); /* base 0 .. (n-3)-1 */
+    else              mpz_isaac_urandomm(base, t);
+    mpz_add_ui(base, base, 2);                          /* base 2 .. n-2     */
     if (miller_rabin(n, base) == 0)
       break;
   }
@@ -855,45 +922,15 @@ int _GMP_is_almost_extra_strong_lucas_pseudoprime(mpz_t n, UV increment)
   return rval;
 }
 
-static void mat_mulmod_3x3(mpz_t* a, mpz_t* b, mpz_t n, mpz_t* t, mpz_t t2) {
-  int i, row, col;
-  for (row = 0; row < 3; row++) {
-    for (col = 0; col < 3; col++) {
-      mpz_mul(t[3*row+col], a[3*row+0], b[0+col]);
-      mpz_mul(t2, a[3*row+1], b[3+col]);
-      mpz_add(t[3*row+col], t[3*row+col], t2);
-      mpz_mul(t2, a[3*row+2], b[6+col]);
-      mpz_add(t[3*row+col], t[3*row+col], t2);
-    }
-  }
-  for (i = 0; i < 9; i++) mpz_mod(a[i], t[i], n);
-}
-static void mat_powmod_3x3(mpz_t* m, mpz_t kin, mpz_t n) {
-  mpz_t k, t2, t[9], res[9];
-  int i;
-  mpz_init_set(k, kin);
-  mpz_init(t2);
-  for (i = 0; i < 9; i++) { mpz_init(t[i]); mpz_init(res[i]); }
-  mpz_set_ui(res[0],1);  mpz_set_ui(res[4],1);  mpz_set_ui(res[8],1);
-  while (mpz_sgn(k)) {
-    if (mpz_odd_p(k))  mat_mulmod_3x3(res, m, n, t, t2);
-    mpz_fdiv_q_2exp(k, k, 1);
-    if (mpz_sgn(k))    mat_mulmod_3x3(m, m, n, t, t2);
-  }
-  for (i = 0; i < 9; i++)
-    { mpz_set(m[i],res[i]);  mpz_clear(res[i]);  mpz_clear(t[i]); }
-  mpz_clear(t2);
-  mpz_clear(k);
-}
 int is_perrin_pseudoprime(mpz_t n, int restricted)
 {
-  int P[9] = {0,1,0, 0,0,1, 1,1,0};
-  mpz_t m[9];
-  int cmpr, i, rval;
+  mpz_t S[6], T[6], T01, T34, T45, t;
+  int cmpr, i, j, rval;
 
   cmpr = mpz_cmp_ui(n, 2);
   if (cmpr == 0)     return 1;  /* 2 is prime */
   if (cmpr < 0)      return 0;  /* below 2 is composite */
+  if (restricted > 2 && mpz_even_p(n)) return 0;
 
   { /* Simple filter for composites */
     uint32_t n32 = mpz_fdiv_ui(n, 2762760);
@@ -904,24 +941,84 @@ int is_perrin_pseudoprime(mpz_t n, int restricted)
     if (!(n32%23) && !((    2 >> (n32%22)) & 1)) return 0;
   }
 
-  for (i = 0; i < 9; i++) mpz_init_set_ui(m[i], P[i]);
-  mat_powmod_3x3(m, n, n);
-  mpz_add(m[1], m[0], m[4]);
-  mpz_add(m[2], m[1], m[8]);
-  mpz_mod(m[0], m[2], n);
-  rval = mpz_sgn(m[0]) ? 0 : 1;
-  if (rval && restricted) {
-    mpz_set_ui(m[0],0);  mpz_set_ui(m[1],1);  mpz_set_ui(m[2],0);
-    mpz_set_ui(m[3],0);  mpz_set_ui(m[4],0);  mpz_set_ui(m[5],1);
-    mpz_set_ui(m[6],1);  mpz_set_ui(m[7],0);  mpz_sub_ui(m[8],n,1);
-    mat_powmod_3x3(m, n, n);
-    mpz_add(m[1], m[0], m[4]);
-    mpz_add(m[2], m[1], m[8]);
-    mpz_add_ui(m[2],m[2],1);
-    mpz_mod(m[0], m[2], n);
-    rval = mpz_sgn(m[0]) ? 0 : 1;
+  /* Calculate signature using Adams/Shanks doubling rule. */
+  mpz_init(t);
+  mpz_init_set_ui(S[0], 1);
+  mpz_init(S[1]); mpz_sub_ui(S[1], n, 1);
+  mpz_init_set_ui(S[2], 3);
+  mpz_init_set_ui(S[3], 3);
+  mpz_init_set_ui(S[4], 0);
+  mpz_init_set_ui(S[5], 2);
+
+  for (i=0; i < 6; i++)
+    mpz_init(T[i]);
+  mpz_init(T01); mpz_init(T34); mpz_init(T45);
+
+  for (i = mpz_sizeinbase(n,2)-2; i >= 0; i--) {
+    mpz_mul(t,S[0],S[0]); mpz_sub(t,t,S[5]); mpz_sub(t,t,S[5]); mpz_mod(T[0],t,n);
+    mpz_mul(t,S[1],S[1]); mpz_sub(t,t,S[4]); mpz_sub(t,t,S[4]); mpz_mod(T[1],t,n);
+    mpz_mul(t,S[2],S[2]); mpz_sub(t,t,S[3]); mpz_sub(t,t,S[3]); mpz_mod(T[2],t,n);
+    mpz_mul(t,S[3],S[3]); mpz_sub(t,t,S[2]); mpz_sub(t,t,S[2]); mpz_mod(T[3],t,n);
+    mpz_mul(t,S[4],S[4]); mpz_sub(t,t,S[1]); mpz_sub(t,t,S[1]); mpz_mod(T[4],t,n);
+    mpz_mul(t,S[5],S[5]); mpz_sub(t,t,S[0]); mpz_sub(t,t,S[0]); mpz_mod(T[5],t,n);
+    mpz_sub(t,T[2],T[1]); mpz_mod(T01,t,n);
+    mpz_sub(t,T[5],T[4]); mpz_mod(T34,t,n);
+    mpz_add(t,T34, T[3]); mpz_mod(T45,t,n);
+    if (mpz_tstbit(n, i)) {
+      mpz_set(S[0],T[0]); mpz_set(S[1],T01); mpz_set(S[2],T[1]);
+      mpz_set(S[3],T[4]); mpz_set(S[4],T45); mpz_set(S[5],T[5]);
+    } else {
+      mpz_add(t,T01,T[0]);
+      mpz_set(S[0],T01); mpz_set(S[1],T[1]); mpz_mod(S[2],t,n);
+      mpz_set(S[3],T34); mpz_set(S[4],T[4]); mpz_set(S[5],T45);
+    }
   }
-  for (i = 0; i < 9; i++) mpz_clear(m[i]);
+
+  for (i=0; i < 6; i++)
+    mpz_clear(T[i]);
+  mpz_clear(T01); mpz_clear(T34); mpz_clear(T45);
+
+  rval = !mpz_sgn(S[4]);
+  if (rval == 0 || restricted == 0)
+    goto DONE_PERRIN;
+
+  mpz_sub_ui(t,n,1);
+  rval = !mpz_cmp(S[1],t);
+  if (rval == 0 || restricted == 1)
+    goto DONE_PERRIN;
+
+  /* Adams/Shanks or Arno,Grantham full signature test */
+  rval = 0;
+  j = mpz_si_kronecker(-23, n);
+
+  if (j == -1) {
+    mpz_t A, B, C;
+    mpz_init_set(B, S[2]); mpz_init(A); mpz_init(C);
+
+    mpz_mul(t,B,B); mpz_mod(t,t,n);
+    mpz_mul_ui(A,B,3); mpz_add_ui(A,A,1); mpz_sub(A,A,t); mpz_mod(A,A,n);
+    mpz_mul_ui(C,t,3); mpz_sub_ui(C,C,2); mpz_mod(C,C,n);
+
+    mpz_mul(t,t,B); mpz_sub(t,t,B); mpz_mod(t,t,n);
+    rval = !mpz_cmp(S[0],A) && !mpz_cmp(S[2],B) && !mpz_cmp(S[3],B) && !mpz_cmp(S[5],C) && mpz_cmp_ui(B,3) && !mpz_cmp_ui(t,1);
+  } else if (restricted > 2 && j == 0 && mpz_cmp_ui(n,23)) {
+    /* Jacobi symbol says 23|n.  n is composite if != 23 */
+    rval = 0;
+  } else {
+    if (!mpz_cmp(S[2],S[3])) {
+      rval = !mpz_cmp_ui(S[0],1) && !mpz_cmp_ui(S[2],3) && !mpz_cmp_ui(S[3],3) && !mpz_cmp_ui(S[5],2);
+    } else {
+      mpz_sub_ui(t, n, 1);
+      rval = !mpz_cmp_ui(S[0],0) && !mpz_cmp(S[5],t) &&
+             (mpz_add(t,S[2],S[3]),mpz_add_ui(t,t,3),mpz_mod(t,t,n),!mpz_sgn(t)) &&
+             (mpz_sub(t,S[2],S[3]),mpz_mul(t,t,t),mpz_add_ui(t,t,23),mpz_mod(t,t,n),!mpz_sgn(t));
+    }
+  }
+
+DONE_PERRIN:
+  for (i=0; i < 6; i++)
+    mpz_clear(S[i]);
+  mpz_clear(t);
   return rval;
 }
 
@@ -1008,7 +1105,6 @@ int is_frobenius_pseudoprime(mpz_t n, IV P, IV Q)
 /* Use Crandall/Pomerance, steps from Loebenberger 2008 */
 int is_frobenius_cp_pseudoprime(mpz_t n, UV ntests)
 {
-  gmp_randstate_t* p_randstate = get_randstate();
   mpz_t t, a, b, d, w1, wm, wm1, m;
   UV tn;
   int j;
@@ -1025,9 +1121,9 @@ int is_frobenius_cp_pseudoprime(mpz_t n, UV ntests)
     /* Step 1: choose a and b in 1..n-1 and d=a^2-4b not square and coprime */
     do {
       mpz_sub_ui(t, n, 1);
-      mpz_urandomm(a, *p_randstate, t);
+      mpz_isaac_urandomm(a, t);
       mpz_add_ui(a, a, 1);
-      mpz_urandomm(b, *p_randstate, t);
+      mpz_isaac_urandomm(b, t);
       mpz_add_ui(b, b, 1);
       /* Check d and gcd */
       mpz_mul(d, a, a);
@@ -1246,7 +1342,7 @@ int _GMP_is_frobenius_khashin_pseudoprime(mpz_t n)
 int _GMP_BPSW(mpz_t n)
 {
   if (mpz_cmp_ui(n, 4) < 0)
-    return (mpz_cmp_ui(n, 1) <= 0) ? 0 : 1;
+    return (mpz_cmp_ui(n, 1) <= 0) ? 0 : 2;
 
   if (miller_rabin_ui(n, 2) == 0)   /* Miller Rabin with base 2 */
     return 0;
@@ -1292,24 +1388,27 @@ int is_deterministic_miller_rabin_prime(mpz_t n)
  *
  * They're all identical for numbers <= 2^64.
  *
- * The extra M-R tests in is_prime start actually costing something after
+ * The extra M-R test in is_prime start actually costing something after
  * 1000 bits or so.  Primality proving will get quite a bit slower as the
  * number of bits increases.
  *
- * What are these extra M-R tests getting us?  The primary reference is
- * Damg??rd, Landrock, and Pomerance, 1993.  From Rabin-Monier, with random
- * bases we have p <= 4^-t.  So one extra test gives us p = 0.25, and four
- * tests gives us p = 0.00390625.  But for larger k (bits in n) this is very
- * conservative.  Since the value has passed BPSW, we are only interested in
- * k > 64.  See the calculate-mr-probs.pl script in the xt/ directory.
- * For a 256-bit input, we get:
- *   1 test:  p < 0.000244140625
- *   2 tests: p < 0.00000000441533
- *   3 tests: p < 0.0000000000062550875
- *   4 tests: p < 0.000000000000028421709
- *
- * It's even more extreme as k goes higher.  Also recall that this is the
- * probability once we've somehow found a BPSW pseudoprime.
+ * Both is_prime and is_provable prime start with some trial division
+ * followed by an extra strong BPSW test, just like is_prob_prime.  For
+ * 65+ bit inputs, they do more.  A single extra random-base M-R test is
+ * next done.  In the worst case this has a 1/4 chance of rejecting the
+ * composite, but an average change of < 1e-12.  is_prime will return at
+ * this point.  is_provable_prime tries various special proofs, followed
+ * by ECPP.  ECPP returns either:
+ *    2  "definitely prime",
+ *    0  "definitely composite, or
+ *    1  "probably prime"
+ * where the latter doesn't really tell us anything.  It's unusual, but
+ * possible.  If this even happs, we do a couple Frobenius-type tests, so
+ * even an answer of 1 (not proven) will have less chance of false results.
+ * A composite would have to have no small factors, be the first known
+ * counterexample to each of three different and distinct tests, pass a
+ * random-base M-R test, and manage to have the ECPP implementation not
+ * find it a composite.
  */
 
 int _GMP_is_prob_prime(mpz_t n)
@@ -1394,37 +1493,27 @@ int _GMP_is_prime(mpz_t n)
    *
    * We will use random bases to make it more difficult to get a false
    * result even if someone has a way to generate BPSW pseudoprimes.
-   * We use enough bases to keep the probability below 1/595,000.
+   *
+   * We can estimate probabilities of random odd 'k'-bit composites passing
+   * 't' random-base Miller-Rabin tests using papers such as Kim and
+   * Pomerance; Damg??rd, Landrock, and Pomerance; Lichtman and Pomerance.
+   * We can also perform random sampling to get empirical data, which shows
+   * a probability of less than 1e-12 at 65 bits, and lowering as the number
+   * of bits increases.  One extra test is all that is necessary.
    */
 
   if (prob_prime == 1) {
-    UV ntests;
-    if      (nbits <  80) ntests = 5;  /* p < .00000168 */
-    else if (nbits < 105) ntests = 4;  /* p < .00000156 */
-    else if (nbits < 160) ntests = 3;  /* p < .00000164 */
-    else if (nbits < 413) ntests = 2;  /* p < .00000156 */
-    else                  ntests = 1;  /* p < .00000159 */
+    UV ntests = 1;
     prob_prime = miller_rabin_random(n, ntests, 0);
     /* prob_prime = _GMP_is_frobenius_underwood_pseudoprime(n); */
     /* prob_prime = _GMP_is_frobenius_khashin_pseudoprime(n); */
   }
 
-  /* Using Damg??rd, Landrock, and Pomerance, we get upper bounds:
-   * k <=  64      p = 0
-   * k <   80      p < 7.53e-08
-   * k <  105      p < 3.97e-08
-   * k <  160      p < 1.68e-08
-   * k >= 160      p < 2.57e-09
-   * This (1) pretends the SPSP-2 is included in the random tests, (2) doesn't
-   * take into account the trial division, (3) ignores the observed
-   * SPSP-2 / Strong Lucas anti-correlation that makes the BPSW test so
-   * useful.  Hence these numbers are still extremely conservative.
-   *
-   * For an idea of how conservative we are being, we have now exceeded the
-   * tests used in Mathematica, Maple, Pari, and SAGE.  Note: Pari pre-2.3
-   * used just M-R tests.  Pari 2.3+ uses BPSW with no extra M-R checks for
-   * is_pseudoprime, but isprime uses APRCL which, being a proof, could only
-   * output a pseudoprime through a coding error.
+  /* We have now done trial division, an SPSP-2 test, an extra-strong Lucas
+   * test, and a random-base Miller-Rabin test.  We have exceeded the testing
+   * done in most math packages.  Any composites will have no small factors,
+   * be the first known BPSW counterexample, and gotten past the random-base
+   * Miller-Rabin test.
    */
 
   return prob_prime;
@@ -1471,5 +1560,13 @@ int _GMP_is_provable_prime(mpz_t n, char** prooftext)
   /* ECPP */
   prob_prime = _GMP_ecpp(n, prooftext);
 
+  /* While extremely unusual, it is not impossible for our ECPP implementation
+   * to give up.  If this happens, we won't return 2 with a proof, but let's
+   * at least run some more tests. */
+  if (prob_prime == 1)
+    prob_prime = _GMP_is_frobenius_underwood_pseudoprime(n);
+  if (prob_prime == 1)
+    prob_prime = _GMP_is_frobenius_khashin_pseudoprime(n);
+
   return prob_prime;
 }
